import { AxiosResponse } from "axios";
import { RequestDefinition, RequestData, ImplementsRequestInterface } from "../requestBase";
import {
  CalendarViewModel,
  CreateCalendarViewModel,
  UpdateCalendarViewModel,
  CalendarTypeViewModel,
  CreateCalendarTypeViewModel,
  UpdateCalendarTypeViewModel,
} from "../../viewmodels/admin/calendar.models";
import { BaseClient } from "../../clients/clientBase";

interface ICalendarRequests {
  getCalendarEntries: RequestDefinition<void, void, CalendarViewModel[]>;
  getCalendarEntryById: RequestDefinition<{ id: string }, void, CalendarViewModel>;
  createCalendarEntry: RequestDefinition<void, CreateCalendarViewModel, string>;
  updateCalendarEntry: RequestDefinition<{ id: string }, UpdateCalendarViewModel, void>;
  deleteCalendarEntry: RequestDefinition<{ id: string }, void, void>;

  getCalendarTypes: RequestDefinition<void, void, CalendarTypeViewModel[]>;
  getCalendarTypeById: RequestDefinition<{ id: number }, void, CalendarTypeViewModel>;
  createCalendarType: RequestDefinition<void, CreateCalendarTypeViewModel, string>;
  updateCalendarType: RequestDefinition<{ id: number }, UpdateCalendarTypeViewModel, void>;
  deleteCalendarType: RequestDefinition<{ id: number }, void, void>;
}

@ImplementsRequestInterface<ICalendarRequests>()
export default abstract class CalendarRequests {
  static async getCalendarEntries({ http }: BaseClient): Promise<AxiosResponse<CalendarViewModel[], any>> {
    return await http.get("/admin/calendar");
  }
  static async getCalendarEntryById({ http }: BaseClient, p: RequestData<{ id: string }, void>): Promise<AxiosResponse<CalendarViewModel, any>> {
    return await http.get(`/admin/calendar/${p.params.id}`);
  }
  static async createCalendarEntry({ http }: BaseClient, p: RequestData<void, CreateCalendarViewModel>): Promise<AxiosResponse<string, any>> {
    return await http.post("/admin/calendar", p.body);
  }
  static async updateCalendarEntry({ http }: BaseClient, p: RequestData<{ id: string }, UpdateCalendarViewModel>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/calendar/${p.params.id}`, p.body);
  }
  static async deleteCalendarEntry({ http }: BaseClient, p: RequestData<{ id: string }, void>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/calendar/${p.params.id}`);
  }

  static async getCalendarTypes({ http }: BaseClient): Promise<AxiosResponse<CalendarTypeViewModel[], any>> {
    return await http.get("/admin/calendartype");
  }
  static async getCalendarTypeById({ http }: BaseClient, p: RequestData<{ id: number }, void>): Promise<AxiosResponse<CalendarTypeViewModel, any>> {
    return await http.get(`/admin/calendartype/${p.params.id}`);
  }
  static async createCalendarType({ http }: BaseClient, p: RequestData<void, CreateCalendarTypeViewModel>): Promise<AxiosResponse<string, any>> {
    return await http.post("/admin/calendartype", p.body);
  }
  static async updateCalendarType(
    { http }: BaseClient,
    p: RequestData<{ id: number }, UpdateCalendarTypeViewModel>
  ): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/calendartype/${p.params.id}`, p.body);
  }
  static async deleteCalendarType({ http }: BaseClient, p: RequestData<{ id: number }, void>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/calendartype/${p.params.id}`);
  }
}
