import { AxiosResponse } from "axios";
import { RequestDefinition, RequestData, ImplementsRequestInterface } from "../requestBase";
import {
  ProtocolViewModel,
  CreateProtocolViewModel,
  SyncProtocolViewModel,
  ProtocolAgendaViewModel,
  ProtocolDecisionViewModel,
  ProtocolPresenceViewModel,
  ProtocolVotingViewModel,
  SyncProtocolAgendaViewModel,
  SyncProtocolDecisionViewModel,
  SyncProtocolPresenceViewModel,
  SyncProtocolVotingViewModel,
} from "../../viewmodels/admin/protocol.models";
import { BaseClient } from "../../clients/clientBase";

interface IProtocolRequests {
  getProtocols: RequestDefinition<void, void, ProtocolViewModel[]>;
  getProtocolById: RequestDefinition<{ id: number }, void, ProtocolViewModel>;
  getProtocolAgendaByProtocol: RequestDefinition<{ protocolId: number }, void, ProtocolAgendaViewModel>;
  getProtocolDecisionsByProtocol: RequestDefinition<{ protocolId: number }, void, ProtocolDecisionViewModel>;
  getProtocolPresenceByProtocol: RequestDefinition<{ protocolId: number }, void, ProtocolPresenceViewModel>;
  getProtocolVotingsByProtocol: RequestDefinition<{ protocolId: number }, void, ProtocolVotingViewModel>;
  createProtocol: RequestDefinition<void, CreateProtocolViewModel, number>;
  createProtocolAgenda: RequestDefinition<{ protocolId: number }, void, number>;
  createProtocolDescision: RequestDefinition<{ protocolId: number }, void, number>;
  createProtocolPresence: RequestDefinition<{ protocolId: number }, void, number>;
  createProtocolVoting: RequestDefinition<{ protocolId: number }, void, number>;
  syncProtocol: RequestDefinition<{ id: number }, SyncProtocolViewModel, void>;
  syncProtocolAgendaByProtocol: RequestDefinition<{ protocolId: number }, SyncProtocolAgendaViewModel, void>;
  syncProtocolDecisionsByProtocol: RequestDefinition<{ protocolId: number }, SyncProtocolDecisionViewModel, void>;
  syncProtocolPresenceByProtocol: RequestDefinition<{ protocolId: number }, SyncProtocolPresenceViewModel, void>;
  syncProtocolVotingsByProtocol: RequestDefinition<{ protocolId: number }, SyncProtocolVotingViewModel, void>;
}

@ImplementsRequestInterface<IProtocolRequests>()
export default abstract class ProtocolRequests {
  static async getProtocols({ http }: BaseClient): Promise<AxiosResponse<ProtocolViewModel[], any>> {
    return await http.get("/admin/protocol");
  }
  static async getProtocolById({ http }: BaseClient, p: RequestData<{ id: number }, void>): Promise<AxiosResponse<ProtocolViewModel, any>> {
    return await http.get(`/admin/protocol/${p.params.id}`);
  }
  static async getProtocolAgendaByProtocol(
    { http }: BaseClient,
    p: RequestData<{ protocolId: number }, void>
  ): Promise<AxiosResponse<ProtocolAgendaViewModel, any>> {
    return await http.get(`/admin/protocol/${p.params.protocolId}/agenda`);
  }
  static async getProtocolDecisionsByProtocol(
    { http }: BaseClient,
    p: RequestData<{ protocolId: number }, void>
  ): Promise<AxiosResponse<ProtocolDecisionViewModel, any>> {
    return await http.get(`/admin/protocol/${p.params.protocolId}/decisions`);
  }
  static async getProtocolPresenceByProtocol(
    { http }: BaseClient,
    p: RequestData<{ protocolId: number }, void>
  ): Promise<AxiosResponse<ProtocolPresenceViewModel, any>> {
    return await http.get(`/admin/protocol/${p.params.protocolId}/presence`);
  }
  static async getProtocolVotingsByProtocol(
    { http }: BaseClient,
    p: RequestData<{ protocolId: number }, void>
  ): Promise<AxiosResponse<ProtocolVotingViewModel, any>> {
    return await http.get(`/admin/protocol/${p.params.protocolId}/votings`);
  }
  static async createProtocol({ http }: BaseClient, p: RequestData<void, CreateProtocolViewModel>): Promise<AxiosResponse<number, any>> {
    return await http.post(`/admin/protocol`, p.body);
  }
  static async createProtocolAgenda({ http }: BaseClient, p: RequestData<{ protocolId: number }, void>): Promise<AxiosResponse<number, any>> {
    return await http.post(`/admin/protocol/${p.params.protocolId}/agenda`);
  }
  static async createProtocolDescision({ http }: BaseClient, p: RequestData<{ protocolId: number }, void>): Promise<AxiosResponse<number, any>> {
    return await http.post(`/admin/protocol/${p.params.protocolId}/decision`);
  }
  static async createProtocolPresence({ http }: BaseClient, p: RequestData<{ protocolId: number }, void>): Promise<AxiosResponse<number, any>> {
    return await http.post(`/admin/protocol/${p.params.protocolId}/presence`);
  }
  static async createProtocolVoting({ http }: BaseClient, p: RequestData<{ protocolId: number }, void>): Promise<AxiosResponse<number, any>> {
    return await http.post(`/admin/protocol/${p.params.protocolId}/voting`);
  }
  static async updateProtocol({ http }: BaseClient, p: RequestData<{ id: number }, SyncProtocolViewModel>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/protocol/${p.params.id}`, p.body);
  }
  static async syncProtocol({ http }: BaseClient, p: RequestData<{ id: number }, SyncProtocolViewModel>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/protocol/${p.params.id}/synchronize`, p.body);
  }
  static async syncProtocolAgendaByProtocol(
    { http }: BaseClient,
    p: RequestData<{ protocolId: number }, SyncProtocolAgendaViewModel>
  ): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/protocol/${p.params.protocolId}/synchronize/agenda`, p.body);
  }
  static async syncProtocolDecisionsByProtocol(
    { http }: BaseClient,
    p: RequestData<{ protocolId: number }, SyncProtocolDecisionViewModel>
  ): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/protocol/${p.params.protocolId}/synchronize/decisions`, p.body);
  }
  static async syncProtocolPresenceByProtocol(
    { http }: BaseClient,
    p: RequestData<{ protocolId: number }, SyncProtocolPresenceViewModel>
  ): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/protocol/${p.params.protocolId}/synchronize/presence`, p.body);
  }
  static async syncProtocolVotingsByProtocol(
    { http }: BaseClient,
    p: RequestData<{ protocolId: number }, SyncProtocolVotingViewModel>
  ): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/protocol/${p.params.protocolId}/synchronize/votings`, p.body);
  }
}
