import { AxiosResponse } from "axios";
import { RequestDefinition, RequestData, ImplementsRequestInterface } from "../requestBase";
import { PermissionObject, PermissionString } from "../../types/admin/permissionTypes";
import { RoleViewModel, UpdateRolePermissionViewModel, UpdateRoleViewModel } from "../../viewmodels/admin/role.models";
import { BaseClient } from "../../clients/clientBase";

interface IRoleRequests {
  getRoles: RequestDefinition<void, void, RoleViewModel[]>;
  getRoleById: RequestDefinition<{ id: number }, void, RoleViewModel>;
  getRolePermissionsById: RequestDefinition<{ roleId: number }, void, PermissionObject>;
  updateRole: RequestDefinition<{ id: number }, UpdateRoleViewModel, void>;
  updateRolePermissions: RequestDefinition<{ roleId: number }, UpdateRolePermissionViewModel, void>;
  deleteRole: RequestDefinition<{ id: number }, void, void>;
}

@ImplementsRequestInterface<IRoleRequests>()
export default abstract class RoleRequests {
  static async getRoles({ http }: BaseClient): Promise<AxiosResponse<RoleViewModel[], any>> {
    return await http.get("/admin/role");
  }
  static async getRoleById({ http }: BaseClient, p: RequestData<{ id: number }, void>): Promise<AxiosResponse<RoleViewModel, any>> {
    return await http.get(`/admin/role/${p.params.id}`);
  }
  static async getRolePermissionsById({ http }: BaseClient, p: RequestData<{ roleId: number }, void>): Promise<AxiosResponse<PermissionObject, any>> {
    return await http.get(`/admin/role/${p.params.roleId}/permissions`);
  }
  static async updateRole({ http }: BaseClient, p: RequestData<{ id: number }, UpdateRoleViewModel>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/role/${p.params.id}`, p.body);
  }
  static async updateRolePermissions(
    { http }: BaseClient,
    p: RequestData<{ roleId: number }, UpdateRolePermissionViewModel>
  ): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/role/${p.params.roleId}/permissions`, p.body);
  }
  static async deleteRole({ http }: BaseClient, p: RequestData<{ id: number }, void>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/role/${p.params.id}`);
  }
}
