import { AxiosResponse } from "axios";
import { RequestDefinition, RequestData, ImplementsRequestInterface } from "../requestBase";
import { CloneTemplateViewModel, CreateTemplateViewModel, TemplateViewModel, UpdateTemplateViewModel } from "../../viewmodels/admin/template.models";
import { TemplateUsageViewModel, UpdateTemplateUsageViewModel } from "../../viewmodels/admin/template.models";
import { BaseClient } from "../../clients/clientBase";
import { PermissionModule } from "../../types/admin/permissionTypes";

interface ITemplateRequests {
  getTemplates: RequestDefinition<void, void, TemplateViewModel[]>;
  getTemplateById: RequestDefinition<{ id: number }, void, TemplateViewModel>;
  createTemplate: RequestDefinition<void, CreateTemplateViewModel, number>;
  cloneTemplate: RequestDefinition<void, CloneTemplateViewModel, number>;
  updateTemplate: RequestDefinition<{ id: number }, UpdateTemplateViewModel, void>;
  deleteTemplate: RequestDefinition<{ id: number }, void, void>;

  getTemplateUsages: RequestDefinition<void, void, TemplateUsageViewModel[]>;
  updateTemplateUsage: RequestDefinition<{ scope: `${PermissionModule}` | `${PermissionModule}.${string}` }, UpdateTemplateUsageViewModel, void>;
}

@ImplementsRequestInterface<ITemplateRequests>()
export default abstract class TemplateRequests {
  static async getTemplates({ http }: BaseClient): Promise<AxiosResponse<TemplateViewModel[], any>> {
    return await http.get("/admin/template");
  }
  static async getTemplateById({ http }: BaseClient, p: RequestData<{ id: number }, void>): Promise<AxiosResponse<TemplateViewModel, any>> {
    return await http.get(`/admin/template/${p.params.id}`);
  }
  static async createTemplate({ http }: BaseClient, p: RequestData<void, CreateTemplateViewModel>): Promise<AxiosResponse<number, any>> {
    return await http.post("/admin/template", p.body);
  }
  static async cloneTemplate({ http }: BaseClient, p: RequestData<void, CloneTemplateViewModel>): Promise<AxiosResponse<number, any>> {
    return await http.post("/admin/template/clone", p.body);
  }
  static async updateTemplate({ http }: BaseClient, p: RequestData<{ id: number }, UpdateTemplateViewModel>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/template/${p.params.id}`, p.body);
  }
  static async deleteTemplate({ http }: BaseClient, p: RequestData<{ id: number }, void>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/template/${p.params.id}`);
  }

  static async getTemplateUsages({ http }: BaseClient): Promise<AxiosResponse<TemplateUsageViewModel[], any>> {
    return await http.get("/admin/templateusage");
  }
  static async updateTemplateUsage(
    { http }: BaseClient,
    p: RequestData<{ scope: `${PermissionModule}` | `${PermissionModule}.${string}` }, UpdateTemplateUsageViewModel>
  ): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/templateusage/${p.params.scope}`, p.body);
  }
}
