import { AxiosResponse } from "axios";
import { RequestDefinition, RequestData, ImplementsRequestInterface } from "../requestBase";
import { BaseClient } from "../../clients/clientBase";
import { Event } from "../../viewmodels/webpage/collection";

interface IEventRequests {
  getEvents: RequestDefinition<void, void, Event[]>;
  getEventById: RequestDefinition<{ id: string }, void, Event>;
  createEvent: RequestDefinition<void, Partial<Event>, string>;
  updateEvent: RequestDefinition<{ id: string }, Partial<Event>, void>;
  deleteEvent: RequestDefinition<{ id: string }, void, void>;
}

@ImplementsRequestInterface<IEventRequests>()
export default abstract class EventRequests {
  static async getEvents({ http }: BaseClient): Promise<AxiosResponse<Event[], any>> {
    return await http.get("/events");
  }
  static async getEventById({ http }: BaseClient, p: RequestData<{ id: string }, void>): Promise<AxiosResponse<Event, any>> {
    return await http.get(`/events/${p.params.id}`);
  }
  static async createEvent({ http }: BaseClient, p: RequestData<void, Partial<Event>>): Promise<AxiosResponse<string, any>> {
    return await http.post("/events", p.body);
  }
  static async updateEvent({ http }: BaseClient, p: RequestData<{ id: string }, Partial<Event>>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/events/${p.params.id}`, p.body);
  }
  static async deleteEvent({ http }: BaseClient, p: RequestData<{ id: string }, void>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/events/${p.params.id}`);
  }
}
