import { AxiosResponse } from "axios";
import { RequestDefinition, RequestData, ImplementsRequestInterface } from "../requestBase";
import { BaseClient } from "../../clients/clientBase";
import { Vehicle } from "../../viewmodels/webpage/collection";

interface IVehicleRequests {
  getVehicles: RequestDefinition<void, void, Vehicle[]>;
  getVehicleById: RequestDefinition<{ id: string }, void, Vehicle>;
  createVehicle: RequestDefinition<void, Partial<Vehicle>, string>;
  updateVehicle: RequestDefinition<{ id: string }, Partial<Vehicle>, void>;
  deleteVehicle: RequestDefinition<{ id: string }, void, void>;
}

@ImplementsRequestInterface<IVehicleRequests>()
export default abstract class VehicleRequests {
  static async getVehicles({ http }: BaseClient): Promise<AxiosResponse<Vehicle[], any>> {
    return await http.get("/vehicles");
  }
  static async getVehicleById({ http }: BaseClient, p: RequestData<{ id: string }, void>): Promise<AxiosResponse<Vehicle, any>> {
    return await http.get(`/vehicles/${p.params.id}`);
  }
  static async createVehicle({ http }: BaseClient, p: RequestData<void, Partial<Vehicle>>): Promise<AxiosResponse<string, any>> {
    return await http.post("/vehicles", p.body);
  }
  static async updateVehicle({ http }: BaseClient, p: RequestData<{ id: string }, Partial<Vehicle>>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/vehicles/${p.params.id}`, p.body);
  }
  static async deleteVehicle({ http }: BaseClient, p: RequestData<{ id: string }, void>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/vehicles/${p.params.id}`);
  }
}
