import { CommunicationTypeViewModel, SalutationViewModel } from "./memberData.models";

/** Member Models */
export interface MemberViewModel {
  id: string;
  salutation: SalutationViewModel;
  firstname: string;
  lastname: string;
  nameaffix: string;
  birthdate: Date;
  internalId?: string;
  firstMembershipEntry?: MembershipViewModel;
  lastMembershipEntry?: MembershipViewModel;
  sendNewsletter?: CommunicationViewModel;
  smsAlarming?: Array<CommunicationViewModel>;
  preferredCommunication?: Array<CommunicationViewModel>;
}
export interface MemberStatisticsViewModel {
  id: string;
  salutation: string;
  firstname: string;
  lastname: string;
  nameaffix: string;
  birthdate: Date;
  todayAge: number;
  ageThisYear: number;
  exactAge: string;
}
export interface CreateMemberViewModel {
  salutationId: number;
  firstname: string;
  lastname: string;
  nameaffix: string;
  birthdate: Date;
  internalId?: string;
}
export interface UpdateMemberViewModel {
  salutationId: number;
  firstname: string;
  lastname: string;
  nameaffix: string;
  birthdate: Date;
  internalId?: string;
}

/** Award Models */
export interface MemberAwardViewModel {
  id: number;
  given: boolean;
  note?: string;
  date: Date;
  award: string;
  awardId: number;
}
export interface CreateMemberAwardViewModel {
  given: boolean;
  note?: string;
  date: Date;
  awardId: number;
}
export interface UpdateMemberAwardViewModel {
  given: boolean;
  note?: string;
  date: Date;
  awardId: number;
}

/** Communication Models */
export interface CommunicationViewModel {
  id: number;
  preferred: boolean;
  mobile: string;
  email: string;
  postalCode: string;
  city: string;
  street: string;
  streetNumber: number;
  streetNumberAddition: string;
  type: CommunicationTypeViewModel;
  isNewsletterMain: boolean;
  isSMSAlarming: boolean;
}
export interface CreateCommunicationViewModel {
  preferred: boolean;
  mobile: string;
  email: string;
  postalCode: string;
  city: string;
  street: string;
  streetNumber: number;
  streetNumberAddition: string;
  typeId: number;
  isNewsletterMain: boolean;
  isSMSAlarming: boolean;
}
export interface UpdateCommunicationViewModel {
  preferred: boolean;
  mobile: string;
  email: string;
  postalCode: string;
  city: string;
  street: string;
  streetNumber: number;
  streetNumberAddition: string;
  isNewsletterMain: boolean;
  isSMSAlarming: boolean;
}

/** Executive Positon Models */
export interface MemberExecutivePositionViewModel {
  id: number;
  note?: string;
  start: Date;
  end?: Date;
  executivePosition: string;
  executivePositionId: number;
}
export interface CreateMemberExecutivePositionViewModel {
  note?: string;
  start: Date;
  end?: Date;
  executivePositionId: number;
}
export interface UpdateMemberExecutivePositionViewModel {
  note?: string;
  start: Date;
  end?: Date;
  executivePositionId: number;
}

/** Qualification Models */
export interface MemberQualificationViewModel {
  id: number;
  note?: string;
  start: Date;
  end?: Date;
  terminationReason?: string;
  qualification: string;
  qualificationId: number;
}
export interface CreateMemberQualificationViewModel {
  note?: string;
  start: Date;
  qualificationId: number;
}
export interface UpdateMemberQualificationViewModel {
  note?: string;
  start: Date;
  end?: Date;
  terminationReason?: string;
  qualificationId: number;
}

/** Membership Models */
export interface MembershipViewModel {
  start: Date;
  end?: Date;
  terminationReason?: string;
  status: string;
  statusId: number;
}
export interface MembershipStatisticsViewModel {
  durationInDays: number;
  durationInYears: string;
  status: string;
  statusId: number;
  memberId: number;
  memberSalutation: string;
  memberFirstname: string;
  memberLastname: string;
  memberNameaffix: string;
  memberBirthdate: Date;
}
export interface CreateMembershipViewModel {
  start: Date;
  statusId: number;
}
export interface UpdateMembershipViewModel {
  start: Date;
  end?: Date;
  terminationReason?: string;
  statusId: number;
}
